@@ -1,11 +1,11 @@
 package OpenTelemetry::SDK;
 # ABSTRACT: An implementation of the OpenTelemetry SDK for Perl
 
-our $VERSION = '0.027';
+our $VERSION = '0.028';
 
 use strict;
 use warnings;
-use experimental qw( signatures lexical_subs );
+use experimental 'signatures';
 use feature 'state';
 
 use Feature::Compat::Try;
@@ -14,10 +14,13 @@ use OpenTelemetry::Common 'config';
 use OpenTelemetry::Propagator::Composite;
 use OpenTelemetry::SDK::Trace::TracerProvider;
 use OpenTelemetry::X;
+use OpenTelemetry;
+use Ref::Util 'is_coderef';
+use Scalar::Util 'blessed';
 
 use isa 'OpenTelemetry::X';
 
-my sub configure_propagators {
+sub configure_propagators ($, @args) {
     my $logger = OpenTelemetry::Common::internal_logger;
 
     state %map = (
@@ -31,16 +34,27 @@ my sub configure_propagators {
         xray         => 'XRay',
     );
 
-    my @names = split ',',
-        ( config('PROPAGATORS') // 'tracecontext,baggage' ) =~ s/\s//gr;
+    push @args, split ',',
+        ( config('PROPAGATORS') // 'tracecontext,baggage' ) =~ s/\s//gr
+        unless @args;
 
     my ( %seen, @propagators );
+    for my $candidate ( grep !!$_, @args ) {
+        if ( blessed $candidate ) {
+            if ( $candidate->DOES('OpenTelemetry::Propagator') ) {
+                push @propagators, $candidate;
+            }
+            else {
+                $logger->warnf("Attempted to configure a '%s' propagator, but it does not do the OpenTelemetry::Propagator role", ref $candidate );
+            }
+            next;
+        }
 
-    for my $name ( @names ) {
-        my $suffix = $map{$name} // do {
-            $logger->warnf("Unknown propagator '%s' cannot be configured", $name);
-            $map{none};
-        };
+        my $suffix = $map{$candidate};
+        unless ($suffix) {
+            $logger->warn("Unknown propagator '$candidate' cannot be configured");
+            next;
+        }
 
         next if $seen{$suffix}++;
 
@@ -52,21 +66,28 @@ my sub configure_propagators {
         }
         catch ($e) {
             die OpenTelemetry::X->create(
-                Invalid => "Error configuring '$name' propagator: $e",
+                Invalid => "Error configuring '$candidate' propagator: $e",
             );
         }
     }
 
+    # If we have no good ones, keep the default
+    return OpenTelemetry->propagator unless @propagators;
+
+    # If we have only one good one, set that one as the global one
+    return OpenTelemetry->propagator = shift @propagators if @propagators == 1;
+
+    # If we have multiple good ones, wrap them in a composite
     OpenTelemetry->propagator
         = OpenTelemetry::Propagator::Composite->new(@propagators),
 }
 
-my sub configure_span_processors {
+sub configure_tracer_provider ($, $provider = undef, @) {
     my $logger = OpenTelemetry::Common::internal_logger;
 
     state %map = (
         jaeger  => '::Jaeger',
-        otlp    => '::OTLP',
+        otlp    => '::OTLP::Traces',
         zipkin  => '::Zipkin',
         console => 'OpenTelemetry::SDK::Exporter::Console',
     );
@@ -74,7 +95,23 @@ my sub configure_span_processors {
     my @names = split ',',
         ( config('TRACES_EXPORTER') // 'otlp' ) =~ s/\s//gr;
 
-    my $tracer_provider = OpenTelemetry::SDK::Trace::TracerProvider->new;
+    if ($provider) {
+        if ( ! blessed $provider ) {
+            $logger->warnf('Attempted to configure a tracer provider that was not a blessed reference: %s', $provider );
+            undef $provider;
+        }
+        elsif (
+            !$provider->can('tracer')
+         || !$provider->can('add_span_processor')
+         || !$provider->can('shutdown')
+         || !$provider->can('force_flush')
+        ) {
+            $logger->warnf("Attempted to configure a '%s' tracer provider, but it does not implement the OpenTelemetry::SDK::Trace::TracerProvider interface", ref $provider );
+            undef $provider;
+        }
+    }
+
+    $provider //= OpenTelemetry::SDK::Trace::TracerProvider->new;
 
     my %seen;
     for my $name ( @names ) {
@@ -98,7 +135,7 @@ my sub configure_span_processors {
             Module::Runtime::require_module $exporter;
             Module::Runtime::require_module $processor;
 
-            $tracer_provider->add_span_processor(
+            $provider->add_span_processor(
                 $processor->new( exporter => $exporter->new )
             );
         }
@@ -109,7 +146,7 @@ my sub configure_span_processors {
         }
     }
 
-    OpenTelemetry->tracer_provider = $tracer_provider;
+    OpenTelemetry->tracer_provider = $provider;
 }
 
 sub import ( $class ) {
@@ -119,8 +156,8 @@ sub import ( $class ) {
         # TODO: logger
         # TODO: error_handler
 
-        configure_propagators();
-        configure_span_processors();
+        configure_propagators(1);
+        configure_tracer_provider(1);
     }
     catch ($e) {
         die $e if isa_OpenTelemetry_X $e;
